@@ -1,5 +1,25 @@
 package SQL::Translator::Parser::DBI::Sybase;
 
+# -------------------------------------------------------------------
+# $Id: Sybase.pm,v 1.4 2003/10/15 16:40:04 kycl4rk Exp $
+# -------------------------------------------------------------------
+# Copyright (C) 2003 Paul Harrington <harringp@deshaw.com>.
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License as
+# published by the Free Software Foundation; version 2.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+# 02111-1307  USA
+# -------------------------------------------------------------------
+
 =head1 NAME
 
 SQL::Translator::Parser::DBI::Sybase - parser for DBD::Sybase
@@ -15,17 +35,17 @@ Uses DBI Catalog Methods.
 =cut
 
 use strict;
-use warnings;
 use DBI;
 use SQL::Translator::Schema;
 use Data::Dumper;
 
-our ( $DEBUG, @EXPORT_OK );
-our $VERSION = '1.59';
+use vars qw[ $DEBUG $VERSION @EXPORT_OK ];
+$VERSION = sprintf "%d.%02d", q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/;
 $DEBUG   = 0 unless defined $DEBUG;
 
 no strict 'refs';
 
+# -------------------------------------------------------------------
 sub parse {
     my ( $tr, $dbh ) = @_;
 
@@ -49,7 +69,7 @@ sub parse {
     # it is much quicker to slurp back everything all at once rather
     # than make repeated calls
 
-    $sth = $dbh->column_info(undef, undef, undef, undef);
+    $sth = $dbh->column_info();
 
 
     foreach my $c (@{$sth->fetchall_arrayref({})}) {
@@ -212,7 +232,7 @@ $table_info->{TABLE_TYPE},
 
             # add in primary key
             my $h = $dbh->selectall_hashref("sp_pkeys
-[$table_info->{TABLE_NAME}]", 'COLUMN_NAME');
+$table_info->{TABLE_NAME}", 'COLUMN_NAME');
             if (scalar keys %{$h} > 1) {
                 my @c = map {
                     $_->{COLUMN_NAME}
@@ -230,7 +250,7 @@ $table_info->{TABLE_TYPE},
 
             if (defined($stuff->{indexes}->{$table_info->{TABLE_NAME}})){
                 my $h = $dbh->selectall_hashref("sp_helpindex
-[$table_info->{TABLE_NAME}]", 'INDEX_NAME');
+$table_info->{TABLE_NAME}", 'INDEX_NAME');
                 foreach (values %{$h}) {
                     my $fields = $_->{'INDEX_KEYS'};
                     $fields =~ s/\s*//g;
@@ -300,6 +320,8 @@ $table_info->{TABLE_NAME},
 
 1;
 
+# -------------------------------------------------------------------
+
 =pod
 
 =head1 AUTHOR
