@@ -1,7 +1,7 @@
 package SQL::Translator::Producer::MySQL;
 
 # -------------------------------------------------------------------
-# $Id: MySQL.pm,v 1.22 2003/06/11 04:00:43 kycl4rk Exp $
+# $Id: MySQL.pm,v 1.28 2003/10/15 18:55:11 kycl4rk Exp $
 # -------------------------------------------------------------------
 # Copyright (C) 2003 Ken Y. Clark <kclark@cpan.org>,
 #                    darren chamberlain <darren@cpan.org>,
@@ -22,9 +22,31 @@ package SQL::Translator::Producer::MySQL;
 # 02111-1307  USA
 # -------------------------------------------------------------------
 
+=head1 NAME
+
+SQL::Translator::Producer::MySQL - MySQL-specific producer for SQL::Translator
+
+=head1 SYNOPSIS
+
+Use via SQL::Translator:
+
+  use SQL::Translator;
+
+  my $t = SQL::Translator->new( parser => '...', producer => 'MySQL', '...' );
+  $t->translate;
+
+=head1 DESCRIPTION
+
+This module will produce text output of the schema suitable for MySQL.
+There are still some issues to be worked out with syntax differences 
+between MySQL versions 3 and 4 ("SET foreign_key_checks," character sets
+for fields, etc.).
+
+=cut
+
 use strict;
 use vars qw[ $VERSION $DEBUG ];
-$VERSION = sprintf "%d.%02d", q$Revision: 1.22 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%02d", q$Revision: 1.28 $ =~ /(\d+)\.(\d+)/;
 $DEBUG   = 0 unless defined $DEBUG;
 
 use Data::Dumper;
@@ -60,6 +82,8 @@ sub produce {
 
     my $create; 
     $create .= header_comment unless ($no_comments);
+    # \todo Don't set if MySQL 3.x is set on command line
+    $create .= "SET foreign_key_checks=0;\n\n";
 
     for my $table ( $schema->get_tables ) {
         my $table_name = $table->name;
@@ -84,6 +108,10 @@ sub produce {
             # data type and size
             my $data_type = $field->data_type;
             my @size      = $field->size;
+            my %extra     = $field->extra;
+            my $list      = $extra{'list'} || [];
+            # \todo deal with embedded quotes
+            my $commalist = join( ', ', map { qq['$_'] } @$list );
 
             #
             # Oracle "number" type -- figure best MySQL type
@@ -107,16 +135,21 @@ sub produce {
                 $data_type = $translate{ $data_type };
             }
 
+            @size = () if $data_type eq 'text';
+
             $field_def .= " $data_type";
-            if ( defined $size[0] && $size[0] > 0 ) {
+            
+            if ( lc $data_type eq 'enum' ) {
+                $field_def .= '(' . $commalist . ')';
+			} 
+            elsif ( defined $size[0] && $size[0] > 0 ) {
                 $field_def .= '(' . join( ', ', @size ) . ')';
             }
 
             # MySQL qualifiers
-            my %extra = $field->extra;
             for my $qual ( qw[ binary unsigned zerofill ] ) {
                 my $val = $extra{ $qual || uc $qual } or next;
-                $field_def .= " $val";
+                $field_def .= " $qual";
             }
 
             # Null?
@@ -143,7 +176,7 @@ sub produce {
         my @index_defs;
         for my $index ( $table->get_indices ) {
             push @index_defs, join( ' ', 
-                $index->type,
+                lc $index->type eq 'normal' ? 'INDEX' : $index->type,
                 $index->name,
                 '(' . join( ', ', $index->fields ) . ')'
             );
@@ -214,13 +247,18 @@ sub produce {
 }
 
 1;
-__END__
 
-=head1 NAME
+# -------------------------------------------------------------------
 
-SQL::Translator::Producer::MySQL - MySQL-specific producer for SQL::Translator
+=pod
+
+=head1 SEE ALSO
+
+SQL::Translator, http://www.mysql.com/.
 
 =head1 AUTHORS
 
 darren chamberlain E<lt>darren@cpan.orgE<gt>,
-Ken Y. Clark E<lt>kclark@cpan.orgE<gt>
+Ken Y. Clark E<lt>kclark@cpan.orgE<gt>.
+
+=cut
